/***************************************************************************
 *            init_client.c
 *
 *  Fri Oct 24 11:26:38 2003
 *  Copyright  2003  Phil Kett
 *  pkett@genesis-midi.com
 ****************************************************************************/

#include "../include/remsync.h"
#include <netdb.h>
#include <sys/socket.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>

int
init_client (int port, char *servername)
{
	int client_sd, set_alarm = 0;
	struct sockaddr_in dest;	//struct for destination address
	struct hostent *hostname;	//struct for destination hostname
	struct in_addr address;	// struct for destination IP address
	char *ip_addr[20], message[BIG_BUF];
	struct sigaction act;
		

	bzero (&dest, sizeof (dest));
	if ((hostname = gethostbyname (servername)) == NULL)
	{
		syslog(LOG_ERR,"Unable to get host %s\n", servername);
		exit (1);
	}
	address.s_addr = *(long *) hostname->h_addr_list[0];	//copy hostname result from gethostbyname into IP address struct
	*ip_addr = inet_ntoa (address);	//convert IP address to string
	if (conf.debug) printf ("IP Addr: %s\n", *ip_addr);
	dest.sin_family = AF_INET;	//set destination address family
	inet_aton (*ip_addr, &dest.sin_addr);	// convert IP address string into format for destination struct
	dest.sin_port = htons (port);	//convert port
	client_sd = socket (AF_INET, SOCK_STREAM, 0);	//create socket client_sd
	if (client_sd == -1)
	{
		syslog(LOG_ERR,"Socket call failed\n");
		exit (1);
	}
	if (connect (client_sd, (struct sockaddr *) &dest, sizeof (dest)) ==	//create connection
	    -1)
	{
		syslog(LOG_ERR,"Connect failed\n");
		exit (1);
	}
	/* set up alarm */
    bzero(&act, sizeof(act));
    act.sa_handler = sig_alarm;
    act.sa_flags = SA_ONESHOT;

    /* Set up handler */
    if (sigaction(SIGALRM, &act, 0) != 0)
		syslog(LOG_ERR,"Unable to set up timeout (init_client)!");
	else 
		set_alarm = 1;

	if (set_alarm == 1) 
		alarm(500); /* 500 seconds timeout */
		
	if (read(client_sd, message, BIG_BUF) < 0) {
		if (errno == EINTR) {
			syslog(LOG_ERR, "Timeout on client connect");
			exit (-1);
		}
	}
	alarm(0); /* clear alarm */
	strip_string(message);
	if (strstr(message, "Remosync") != NULL) {
		if (strcmp(message, VERSTR) != 0) {
			syslog(LOG_ERR,"Not the same version of Remosync - Exiting...");
			exit(-1);
		}
	} else {
		printf("%s\n", message);
		exit(-1);
	}
	return client_sd;	//return file descriptor of connection
}
